package org.foxarmy.barcodescannerforemployees

import android.content.Context
import android.database.sqlite.SQLiteDatabase
import android.database.sqlite.SQLiteOpenHelper
import android.provider.BaseColumns
import java.io.File

object ProductContract {
    object ProductEntry : BaseColumns {
        const val TABLE_NAME = "products"
        const val PRODUCT_NAME = "name"
        const val PRODUCT_NET_WEIGHT = "net_weight"
        const val IMAGE_FILENAME = "image_filename"
        const val CATEGORY = "category"
    }
}

object ShelfContract {
    object ShelfEntry : BaseColumns {
        const val TABLE_NAME = "shelf"
        const val PRODUCT_ID = "product_id"
        const val EXPIRE_DATE = "expire_date"
    }
}

object CategoriesContract {
    object CategoryEntry : BaseColumns {
        const val TABLE_NAME = "categories"
        const val CATEGORY_NAME = "category_name"
    }
}

const val SQL_CREATE_PRODUCT_TABLE =
    "CREATE TABLE ${ProductContract.ProductEntry.TABLE_NAME} (" +
            "${BaseColumns._ID} INTEGER PRIMARY KEY," +
            "${ProductContract.ProductEntry.PRODUCT_NAME} TEXT," +
            "${ProductContract.ProductEntry.PRODUCT_NET_WEIGHT} REAL," +
            "${ProductContract.ProductEntry.IMAGE_FILENAME} TEXT," +
            "${ProductContract.ProductEntry.CATEGORY} INTEGER)"

const val SQL_CREATE_SHELF_TABLE =
    "CREATE TABLE ${ShelfContract.ShelfEntry.TABLE_NAME} (" +
        "${BaseColumns._ID} INTEGER PRIMARY KEY," +
        "${ShelfContract.ShelfEntry.PRODUCT_ID} INTEGER," +
        "${ShelfContract.ShelfEntry.EXPIRE_DATE} DATE)"

const val SQL_CREATE_CATEGORIES_TABLE =
    "CREATE TABLE ${CategoriesContract.CategoryEntry.TABLE_NAME} (" +
            "${BaseColumns._ID} INTEGER PRIMARY KEY," +
            "${CategoriesContract.CategoryEntry.CATEGORY_NAME} TEXT)"

class DBStorageController(context: Context) : SQLiteOpenHelper(context, DATABASE_NAME, null, DATABASE_VERSION) {

    override fun onCreate(db: SQLiteDatabase) {
        db.execSQL(SQL_CREATE_PRODUCT_TABLE)
        db.execSQL(SQL_CREATE_SHELF_TABLE)
        db.execSQL(SQL_CREATE_CATEGORIES_TABLE)
    }

    override fun onUpgrade(db: SQLiteDatabase?, oldV: Int, newV: Int) {
        TODO("Not yet implemented")
    }

    fun eraseCategory (db: SQLiteDatabase, id: Int, context: Context) {
        val productsInCategory = getAllAbstractProductInCategory(db, id)

        for (product in productsInCategory.iterator()) {
            eraseAbstractProduct(db, product.id, context)
        }

        db.delete(CategoriesContract.CategoryEntry.TABLE_NAME, BaseColumns._ID + "=" + id, null)
    }

    fun getCategoryNameById(db: SQLiteDatabase, id: Int) : String {
        var result = ""

        val projection = arrayOf(CategoriesContract.CategoryEntry.CATEGORY_NAME)
        val selection = "${BaseColumns._ID} = ?"
        val selectionArgs = arrayOf(id.toString())

        val cursor = db.query(CategoriesContract.CategoryEntry.TABLE_NAME, projection, selection, selectionArgs, null, null, null)

        with (cursor) {
            while (moveToNext()) {
                result = getString(getColumnIndexOrThrow(CategoriesContract.CategoryEntry.CATEGORY_NAME))
            }
        }

        return result
    }

    fun getAllAbstractProductInCategory(db: SQLiteDatabase, id: Int) : List<AbstractProduct> {

        var result = mutableListOf<AbstractProduct>()
        val projection = arrayOf(
            BaseColumns._ID,
            ProductContract.ProductEntry.PRODUCT_NAME,
            ProductContract.ProductEntry.IMAGE_FILENAME,
            ProductContract.ProductEntry.PRODUCT_NET_WEIGHT,
        )

        val selection = "${ProductContract.ProductEntry.CATEGORY} = ?"
        val selectionArgs = arrayOf(id.toString())

        val cursor = db.query(ProductContract.ProductEntry.TABLE_NAME, projection, selection, selectionArgs, null, null, null)

        with(cursor) {
            while (moveToNext()) {
                val abstractProductId = getInt(getColumnIndexOrThrow(BaseColumns._ID))
                val abstractProductName = getString(getColumnIndexOrThrow(ProductContract.ProductEntry.PRODUCT_NAME))
                val abstractProductNetWeight = getDouble(getColumnIndexOrThrow(ProductContract.ProductEntry.PRODUCT_NET_WEIGHT))
                val abstractProductImageHash = getString(getColumnIndexOrThrow(ProductContract.ProductEntry.IMAGE_FILENAME))

                val abstractProduct = AbstractProduct(abstractProductId, abstractProductName, abstractProductNetWeight, abstractProductImageHash, category = id)

                result.add(abstractProduct)
            }
        }

        return result
    }

    fun eraseAbstractProduct(db: SQLiteDatabase, id: Int, context: Context) {
        val projection = arrayOf(
            ProductContract.ProductEntry.IMAGE_FILENAME
        )

        val selection = "${BaseColumns._ID} = ?"
        val selectionArgs = arrayOf(id.toString())
        val cursor = db.query(
            ProductContract.ProductEntry.TABLE_NAME,
            projection,
            selection,
            selectionArgs,
            null,
            null,
            null
        )
        var imageHash: String = ""

        with (cursor) {
            while(moveToNext()) {
                val productImageHash = getString(getColumnIndexOrThrow(ProductContract.ProductEntry.IMAGE_FILENAME))
                imageHash = productImageHash
            }
        }
        val picturesDir = File(context.filesDir, "pictures")
        val thumbnailsDir = File(context.cacheDir, "thumbnails")

        File(picturesDir, "$imageHash.png").delete()
        File(thumbnailsDir, "$imageHash.webp").delete()
        db.delete(ProductContract.ProductEntry.TABLE_NAME, BaseColumns._ID + "=" + id, null)
    }

    companion object {
        const val DATABASE_VERSION = 1
        const val DATABASE_NAME = "database.db"
    }
}