package org.foxarmy.barcodescannerforemployees.activities

import android.content.DialogInterface
import android.content.Intent
import android.os.Bundle
import android.view.Menu
import android.view.MenuItem
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import androidx.core.content.ContextCompat
import androidx.viewpager.widget.ViewPager
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.ViewPagerAdapter
import org.foxarmy.barcodescannerforemployees.databinding.ActivityMainBinding
import org.foxarmy.barcodescannerforemployees.fragments.CategoriesFragment
import org.foxarmy.barcodescannerforemployees.fragments.StorageFragment

class MainActivity : AppCompatActivity() {

    private lateinit var binding: ActivityMainBinding
    public lateinit var adapter: ViewPagerAdapter

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)

        setSupportActionBar(binding.toolbar)
        setupViewPager(binding.tabViewpager)
        binding.tabTablayout.setupWithViewPager(binding.tabViewpager)

        binding.newElementFab.setOnClickListener { view ->
            val currentPosition = binding.tabTablayout.selectedTabPosition
            val fragment = adapter.getItem(currentPosition)

            when (fragment::class.simpleName.toString()) {
                "StorageFragment" -> {
                    val addProductIntent = Intent(this, AddProductActivity::class.java)
                    val extras = Bundle()
                    // I reuse the same stuff for editing and adding new product.
                    // if updatingObject == null, it means that we need to create new object
                    extras.putParcelable("updatingObject", null)
                    addProductIntent.putExtras(extras)
                    ContextCompat.startActivity(this, addProductIntent, extras)
                }

                "CategoriesFragment" -> {
                    val addCategoryIntent = Intent(this, AddCategoryActivity::class.java)
                    val extras = Bundle()
                    extras.putInt("categoryid", 0)
                    extras.putString("categoryname", "New category")
                    addCategoryIntent.putExtras(extras)
                    ContextCompat.startActivity(this, addCategoryIntent, extras)
                }
            }

        }
    }

    private fun setupViewPager(viewpager: ViewPager) {
        adapter = ViewPagerAdapter(supportFragmentManager)

        adapter.addFragment(CategoriesFragment(), "Categories")
        adapter.addFragment(StorageFragment(), "Storage")
        //TODO: shelf and settings fragments

        // setting adapter to view pager.
        viewpager.setAdapter(adapter)
    }

    override fun onCreateOptionsMenu(menu: Menu): Boolean {
        menuInflater.inflate(R.menu.menu_main, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        val currentPosition = binding.tabTablayout.selectedTabPosition
        val fragment = adapter.getItem(currentPosition)

        return when (item.itemId) {


            R.id.action_settings -> {
                true
            }

            R.id.action_delete -> {

                when (fragment::class.simpleName.toString()) {
                    "StorageFragment" -> {
                        val storageFragment = fragment as StorageFragment
                        storageFragment.removeSelected()
                    }

                    "CategoriesFragment" -> {
                        AlertDialog.Builder(this)
                            .setMessage("Deleting a category will also delete ALL the products, that belong to that category. Do you want to proceed?")
                            .setPositiveButton("Yes") { _: DialogInterface, _: Int ->
                                val categoriesFragment = fragment as CategoriesFragment
                                categoriesFragment.removeSelected()
                            }
                            .setNegativeButton("No") { _: DialogInterface, _: Int ->

                            }.show()
                    }
                }
                true
            }

            R.id.action_update -> {
                when (fragment::class.simpleName.toString()) {
                    "StorageFragment" -> {
                        val storageFragment = fragment as StorageFragment
                        storageFragment.updateSelected()
                    }

                    "CategoriesFragment" -> {
                        val categoriesFragment = fragment as CategoriesFragment
                        categoriesFragment.updateSelected()
                    }
                }
                true
            }

            else -> super.onOptionsItemSelected(item)
        }
    }
}