package org.foxarmy.barcodescannerforemployees.fragments

import android.content.Intent
import android.os.Bundle
import android.provider.BaseColumns
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.Toast
import androidx.core.content.ContextCompat
import androidx.core.view.children
import androidx.fragment.app.Fragment
import androidx.gridlayout.widget.GridLayout
import org.foxarmy.barcodescannerforemployees.*
import org.foxarmy.barcodescannerforemployees.activities.AddProductActivity
import org.foxarmy.barcodescannerforemployees.views.AbstractProductView

class StorageFragment : Fragment() {

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        return inflater.inflate(R.layout.fragment_storage, container, false)
    }

    override fun onResume() {
        super.onResume()

        updateContent()
    }

    fun removeSelected() {
        val grv = view?.findViewById<GridLayout>(R.id.contentGridLayout)

        val db = DBStorageController(requireContext())
        var deleted = false
        for (view: AbstractProductView in grv?.children!!.iterator() as Iterator<AbstractProductView>) {
            view.findViewById<ImageView>(R.id.productPicture).setImageURI(null)
            if (view.isProductSelected) {
                db.eraseAbstractProduct(db.writableDatabase, view.product.id, requireContext())
                deleted = true
            }
        }

        if (!deleted) {
            Toast.makeText(requireContext(), "Nothing to delete", Toast.LENGTH_SHORT).show()
        }
        updateContent()
    }

    fun updateSelected() {
        val grv = view?.findViewById<GridLayout>(R.id.contentGridLayout)

        for (view: AbstractProductView in grv?.children!!.iterator() as Iterator<AbstractProductView>) {
            if (view.isProductSelected) {
                val addProductIntent = Intent(requireContext(), AddProductActivity::class.java)
                val extras = Bundle()
                extras.putParcelable("updatingObject", view.product)
                addProductIntent.putExtras(extras)
                ContextCompat.startActivity(requireContext(), addProductIntent, extras)
            }
        }

    }

    fun updateContent() {

        val grv = view?.findViewById<GridLayout>(R.id.contentGridLayout)
        grv?.removeAllViews()

        val db = DBStorageController(requireContext()).readableDatabase
        val projection = arrayOf(BaseColumns._ID,
            ProductContract.ProductEntry.PRODUCT_NAME,
            ProductContract.ProductEntry.PRODUCT_NET_WEIGHT,
            ProductContract.ProductEntry.IMAGE_FILENAME,
            ProductContract.ProductEntry.CATEGORY
        )

        val cursor = db.query(ProductContract.ProductEntry.TABLE_NAME, projection, null, null, null, null, null)

        with (cursor) {
            while(moveToNext()) {
                val productId = getInt(getColumnIndexOrThrow(BaseColumns._ID))
                val productName = getString(getColumnIndexOrThrow(ProductContract.ProductEntry.PRODUCT_NAME))
                val netWeight = getDouble(getColumnIndexOrThrow(ProductContract.ProductEntry.PRODUCT_NET_WEIGHT))
                val productImageHash = getString(getColumnIndexOrThrow(ProductContract.ProductEntry.IMAGE_FILENAME))
                val category = getInt(getColumnIndexOrThrow(ProductContract.ProductEntry.CATEGORY))

                val product = AbstractProduct(productId, productName, netWeight, productImageHash, category)

                generateThumbnailForImage(context!!, productImageHash)

                val abstractProduct = AbstractProductView(
                    requireActivity(),
                    requireContext(),
                    product
                )
                grv?.addView(abstractProduct)
            }
        }
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        updateContent()
    }
}