package org.foxarmy.barcodescannerforemployees.views

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.core.content.ContextCompat
import androidx.core.content.ContextCompat.startActivity
import org.foxarmy.barcodescannerforemployees.AbstractProduct
import org.foxarmy.barcodescannerforemployees.DBStorageController
import org.foxarmy.barcodescannerforemployees.R
import org.foxarmy.barcodescannerforemployees.activities.FullscreenActivity
import org.foxarmy.barcodescannerforemployees.getImageUri
import java.io.File

class AbstractProductView: LinearLayout {
    private var productLayout: ConstraintLayout
    private var productPicture: ImageView
    private var productNameField: TextView
    private var netWeightField: TextView
    private var categoryField: TextView
    private var unitField: TextView
    var product: AbstractProduct
    var isProductSelected = false

    constructor(activity: Activity, context: Context, product: AbstractProduct) : super(context) {

        this.product = product

        val inflater:LayoutInflater = activity.layoutInflater
        inflater.inflate(R.layout.abstract_product_view, this)

        productLayout = findViewById(R.id.productLayout)
        productPicture = findViewById(R.id.productPicture)
        productNameField = findViewById(R.id.productNameView)
        netWeightField = findViewById(R.id.productNetWeightView)
        categoryField = findViewById(R.id.categoryView)
        unitField = findViewById(R.id.unitView)

        val thumbnailsDir = File(context.cacheDir, "thumbnails")
        thumbnailsDir.mkdirs()
        val imageUri = getImageUri(activity, File(thumbnailsDir, "${product.imageHash}.webp"))
        productPicture.setImageURI(imageUri)
        productPicture.rotation = 90f
        productPicture.setOnClickListener {
            val fullscreenIntent = Intent(activity, FullscreenActivity::class.java)
            val extras = Bundle()
            extras.putString("imagehash", product.imageHash)
            fullscreenIntent.putExtras(extras)
            startActivity(context, fullscreenIntent, extras)
        }

        productLayout.setOnClickListener {

        }



        productNameField.text = product.name
        netWeightField.text = product.netWeight.toString()
        categoryField.text = DBStorageController(context).getCategoryNameById(DBStorageController(context).readableDatabase, product.category)

        //TODO: units

        productLayout.setOnLongClickListener {
            isProductSelected = !isProductSelected
            this.background = ContextCompat.getDrawable(context, if (isProductSelected) R.drawable.outline_selected else R.drawable.outline)
            true
        }
    }
}